@@ -10,7 +10,7 @@
 
 ;;; Code:
 
-(defvar agda2-version "2.5.4"
+(defvar agda2-version "2.6.0"
   "The version of the Agda mode.
 Note that the same version of the Agda executable must be used.")
 
@@ -185,19 +185,6 @@ to this variable to take effect."
 (if (and (equal agda2-fontset-name "fontset-agda2") window-system)
     (create-fontset-from-fontset-spec agda2-fontset-spec-of-fontset-agda2 t t))
 
-(defcustom agda2-highlight-after-inactivity-interval 0.2
-  "Highlight the buffer when Emacs has been idle for the given time.
-After the last modification of the buffer. Only token-based
-highlighting is performed. Highlighting is only performed if the
-buffer is (marked as being) modified, the Agda process is not
-busy, and `agda2-highlight-level' is not `none'."
-  :group 'agda2-highlight
-  :type '(choice (const :tag "Turn off this feature" nil)
-                 (restricted-sexp
-                  :tag "Seconds"
-                  :match-alternatives ((lambda (n)
-                                         (and (numberp n) (>= n 0)))))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;; Global and buffer-local vars, initialization
 
@@ -250,7 +237,7 @@ constituents.")
     (agda2-previous-goal                     "\C-c\C-b"           (global)       "Previous goal") ; Back.
     (agda2-give                              ,(kbd "C-c C-SPC")   (local)        "Give")
     (agda2-refine                            "\C-c\C-r"           (local)        "Refine")
-    (agda2-auto                              "\C-c\C-a"           (local)        "Auto")
+    (agda2-auto-maybe-all                    "\C-c\C-a"           (local global) "Auto")
     (agda2-make-case                         "\C-c\C-c"           (local)        "Case")
     (agda2-goal-type                         "\C-c\C-t"           (local)        "Goal type")
     (agda2-show-context                      "\C-c\C-e"           (local)        "Context (environment)")
@@ -395,10 +382,6 @@ Note that this variable is not buffer-local.")
   "Was `agda2-file-buffer' active when `agda2-output-filter' started?
 Note that this variable is not buffer-local.")
 
-(defvar agda2-highlight-after-inactivity-timer nil
-  "Timer used to update token-based highlighting after inactivity.")
-(make-variable-buffer-local 'agda2-highlight-after-inactivity-timer)
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;; agda2-mode
 
@@ -447,7 +430,7 @@ agda2-include-dirs is not bound." :warning))
        (error (error "Unable to change the font; change agda2-fontset-name or tweak agda2-fontset-spec-of-fontset-agda2"))))
  ;; Deactivate highlighting if the buffer is edited before
  ;; typechecking is complete.
- (add-hook 'after-change-functions 'agda2-abort-highlighting nil 'local)
+ (add-hook 'first-change-hook 'agda2-abort-highlighting nil 'local)
  ;; If Agda is not running syntax highlighting does not work properly.
  (unless (eq 'run (agda2-process-status))
    (agda2-restart))
@@ -759,9 +742,9 @@ reloaded from `agda2-highlighting-file', unless
   (unless agda2-in-progress
       (setq agda2-highlight-in-progress nil)))
 
-(defun agda2-abort-highlighting (beg end len)
+(defun agda2-abort-highlighting nil
   "Abort any interactive highlighting.
-This function should be used in `after-change-functions'."
+This function should be used in `first-change-hook'."
   (when agda2-highlight-in-progress
     (setq agda2-highlight-in-progress nil)
     (message "\"%s\" has been modified. Interrupting highlighting."
@@ -930,9 +913,15 @@ of new goals."
       (agda2-goal-cmd "Cmd_refine_or_intro True" 'save 'goal)
     (agda2-goal-cmd "Cmd_refine_or_intro False" 'save 'goal)))
 
-(defun agda2-auto ()
+(defun agda2-autoOne ()
  "Simple proof search" (interactive)
- (agda2-goal-cmd "Cmd_auto" 'save 'goal))
+ (agda2-goal-cmd "Cmd_autoOne" 'save 'goal))
+
+(defun agda2-autoAll ()
+  (interactive)
+  "Solves all goals by simple proof search."
+  (agda2-go nil t nil t "Cmd_autoAll")
+)
 
 (defun agda2-make-case ()
   "Refine the pattern variables given in the goal.
@@ -1228,11 +1217,7 @@ is inserted, and point is placed before this text."
 (defun agda2-quit ()
   "Quit and clean up after agda2."
   (interactive)
-  (when (timerp agda2-highlight-after-inactivity-timer)
-    (cancel-timer agda2-highlight-after-inactivity-timer))
-  (remove-hook 'after-change-functions
-               'agda2-highlight-after-inactivity 'local)
-  (remove-hook 'after-change-functions 'agda2-abort-highlighting 'local)
+  (remove-hook 'first-change-hook 'agda2-abort-highlighting 'local)
   (remove-hook 'after-save-hook 'agda2-highlight-tokens 'local)
   (agda2-remove-annotations)
   (agda2-term))
@@ -1457,6 +1442,15 @@ Either only one if point is a goal, or all of them."
                           'agda2-solveAll))
 )
 
+(defun agda2-auto-maybe-all ()
+  "Run auto.
+Either only one if point is a goal, or all of them."
+  (interactive)
+  (call-interactively (if (agda2-goal-at (point))
+                          'agda2-autoOne
+                          'agda2-autoAll))
+)
+
 (agda2-maybe-normalised-global
   agda2-solveAll
   "Solves all goals that are already instantiated internally."
@@ -1859,12 +1853,8 @@ a file is loaded."
   ;; things). Syntax table setup for comments is done elsewhere.
   (set (make-local-variable 'comment-use-syntax) t)
 
-  ;; Update token-based highlighting after the buffer has been saved
-  ;; and after Emacs has been idle for a certain amount of time (and
-  ;; the buffer has been modified).
+  ;; Update token-based highlighting after the buffer has been saved.
   (add-hook 'after-save-hook 'agda2-highlight-tokens nil 'local)
-  (add-hook 'after-change-functions 'agda2-highlight-after-inactivity
-            nil 'local)
 
   ;; Support for proper filling of text in comments (requires that
   ;; Filladapt is activated).
@@ -1890,59 +1880,17 @@ From the beginning of the current line to the end of the buffer."
           (comment-dwim nil))
       (pop-mark))))
 
-(defun agda2-all-extensions (filename)
-  "Return all of FILENAME's extensions.
-
-FILENAME must not contain any extensions treated specially by
-`file-name-extension', such as ~3~."
-  (let ((ext     (concat "." (file-name-extension filename)))
-        (newname (file-name-sans-extension filename)))
-    (if (equal newname filename)
-        ""
-      (concat (agda2-all-extensions newname) ext))))
-
 (defun agda2-highlight-tokens nil
   "Compute token-based highlighting information.
 
 Unless `agda2-highlight-level' is `none' or the Agda process is
-busy with something. This command does not save the buffer."
+busy with something. This command might save the buffer."
   (unless (or agda2-in-progress
               (equal agda2-highlight-level 'none))
-    (let ((tmp (make-temp-file "agda2-" nil
-                               (agda2-all-extensions
-                                (file-name-nondirectory
-                                 (buffer-file-name))))))
-      (write-region nil nil tmp nil 'do-not-display-message)
-      (agda2-go 'do-not-save nil t t
-                "Cmd_tokenHighlighting"
-                (agda2-string-quote tmp)
-                "Remove"))))
-
-(defun agda2-highlight-after-inactivity (beg end len)
-  "Compute token-based highlighting after Emacs has been idle.
-Only if `agda2-highlight-after-inactivity-interval' is a
-non-negative number. The number is interpreted as the number of
-seconds of continuous inactivity that is required before
-highlighting is updated. Furthermore nothing happens unless,
-after the given amount of time, the buffer is marked as being
-modified, the Agda process is not busy, and
-`agda2-highlight-level' is not `none'.
-
-When this procedure is invoked previous, pending invocations are
-cancelled.
-
-This procedure is intended to be used in `after-change-functions'."
-  (when (and (numberp agda2-highlight-after-inactivity-interval)
-             (>= agda2-highlight-after-inactivity-interval 0))
-    (when (timerp agda2-highlight-after-inactivity-timer)
-      (cancel-timer agda2-highlight-after-inactivity-timer))
-    (setq agda2-highlight-after-inactivity-timer
-          (run-with-idle-timer agda2-highlight-after-inactivity-interval
-                               nil
-                               `(lambda nil
-                                 (with-current-buffer ,(current-buffer)
-                                   (when (buffer-modified-p)
-                                     (agda2-highlight-tokens))))))))
+    (agda2-go 'save nil t t
+              "Cmd_tokenHighlighting"
+              (agda2-string-quote (buffer-file-name))
+              "Keep")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Go to definition site
